from typing import List, Dict, Type
from pydantic import BaseModel, Field
from langchain.tools import BaseTool
from typing_extensions import Annotated
import requests

class BatteryStatusRequest(BaseModel):
    robot_ids: Annotated[List[str], Field(description="로봇들의 ID 목록입니다. 여러 개의 로봇을 동시에 조회할 수 있습니다.")]

class BatteryStatusResponse(BaseModel):
    robot_id: Annotated[str, Field(description="배터리 상태를 조회할 로봇의 ID입니다.")]
    battery_level: Annotated[int, Field(description="로봇의 배터리 잔여량(%)입니다.")]

class CheckRobotBatteryStatusTool(BaseTool):
    """
    로봇 배터리 상태를 확인하는 툴.

    지정된 로봇들의 배터리 상태를 확인합니다. 여러 로봇의 배터리 레벨을 동시에 조회하여, 
    각 로봇의 배터리 상태를 신속하게 파악할 수 있습니다. 이 툴은 로봇의 배터리 잔여량을 기반으로 상태를 
    모니터링하고, 배터리 관리가 필요한 로봇들을 추적하는 데 유용합니다.

    """
    name : str = "로봇 배터리 상태 확인"
    description : str = "지정된 로봇들의 배터리 상태를 확인합니다. 여러 로봇의 배터리 레벨을 동시에 조회하여,"\
    "각 로봇의 배터리 상태를 신속하게 파악할 수 있습니다." \
    "이 툴은 로봇의 배터리 잔여량을 기반으로 상태를 모니터링하고," \
    "배터리 관리가 필요한 로봇들을 추적하는 데 유용합니다."
    
    metadata : Dict[str,str] = {"category": "robot_status"}
    
    args_schema : Type[BaseModel] = BatteryStatusRequest
    """
    이 툴은 로봇들의 배터리 상태를 조회하기 위한 요청 데이터를 정의합니다.
    
    - `robot_ids`: 배터리 상태를 확인할 로봇들의 ID 목록입니다.
    """

    def _run(self, robot_ids: List[str]) -> List[BatteryStatusResponse]:
        """
        로봇 배터리 상태를 조회합니다.

        지정된 로봇들의 배터리 상태를 확인하고, 그 결과를 `BatteryStatusResponse` 형식으로 반환합니다.

        Args:
            robot_ids (List[str]): 배터리 상태를 조회할 로봇들의 ID 목록입니다.

        Returns:
            List[BatteryStatusResponse]: 각 로봇의 ID와 배터리 상태를 포함하는 리스트입니다.
        """
        url = "http://localhost:8000/check_robot_battery_statuses"
        response = requests.post(url, json={"robot_ids": robot_ids})
        
        if response.status_code == 200:
            return [
                BatteryStatusResponse(robot_id=robot['robot_id'], battery_level=robot['battery_level'])
                for robot in response.json()
            ]
        else:
            return []



    async def _arun(self, robot_ids: List[str]) -> List[BatteryStatusResponse]:
        """
        비동기 방식으로 로봇 배터리 상태를 조회합니다.

        비동기 요청을 사용하여 지정된 로봇들의 배터리 상태를 확인합니다.

        Args:
            robot_ids (List[str]): 배터리 상태를 조회할 로봇들의 ID 목록입니다.

        Returns:
            List[BatteryStatusResponse]: 각 로봇의 ID와 배터리 상태를 포함하는 리스트입니다.
            
        """
        # 비동기 방식으로 처리할 경우, 해당 로직을 구현합니다.
        pass

